@@ -2,7 +2,7 @@
 My own variation on function-specific inspect-like features.
 """
 
-# Author: Gael Varoquaux <gael dot varoquaux at normalesup dot org> 
+# Author: Gael Varoquaux <gael dot varoquaux at normalesup dot org>
 # Copyright (c) 2009 Gael Varoquaux
 # License: BSD Style, 3 clauses.
 
@@ -11,9 +11,10 @@ import inspect
 import warnings
 import os
 
+
 def get_func_code(func):
     """ Attempts to retrieve a reliable function code hash.
-    
+
         The reason we don't use inspect.getsource is that it caches the
         source, whereas we want this to be modified on the fly when the
         function is modified.
@@ -39,7 +40,8 @@ def get_func_code(func):
         source_file_obj = file(source_file)
         first_line = func.func_code.co_firstlineno
         # All the lines after the function definition:
-        source_lines = list(itertools.islice(source_file_obj, first_line-1, None))
+        source_lines = list(itertools.islice(source_file_obj, first_line - 1,
+                                             None))
         return ''.join(inspect.getblock(source_lines)), source_file, first_line
     except:
         # If the source code fails, we use the hash. This is fragile and
@@ -54,7 +56,15 @@ def get_func_code(func):
             return repr(func), source_file, -1
 
 
-def get_func_name(func, resolv_alias=True, win_characters=True):  
+def _clean_win_chars(string):
+    "Windows cannot encode some characters in filenames"
+    import urllib
+    for char in ('<', '>', '!', ':', '\\'):
+        string = string.replace(char, urllib.quote(char))
+    return string
+
+
+def get_func_name(func, resolv_alias=True, win_characters=True):
     """ Return the function import path (as a list of module names), and
         a name for the function.
 
@@ -83,18 +93,20 @@ def get_func_name(func, resolv_alias=True, win_characters=True):
         module = ''
     if module == '__main__':
         try:
-            filename = inspect.getsourcefile(func)
+            filename = os.path.abspath(inspect.getsourcefile(func))
         except:
             filename = None
         if filename is not None:
-            filename = filename.replace('/', '-')
+            # mangling of full path to filename
+            filename = filename.replace(os.sep, '-')
+            filename = filename.replace(":", "-")
             if filename.endswith('.py'):
                 filename = filename[:-3]
             module = module + '-' + filename
     module = module.split('.')
     if hasattr(func, 'func_name'):
         name = func.func_name
-    elif hasattr(func, '__name__'): 
+    elif hasattr(func, '__name__'):
         name = func.__name__
     else:
         name = 'unknown'
@@ -105,15 +117,14 @@ def get_func_name(func, resolv_alias=True, win_characters=True):
             if not func.func_globals[name] is func:
                 name = '%s-alias' % name
     if inspect.ismethod(func):
-        # We need to add the name of the class    
+        # We need to add the name of the class
         if hasattr(func, 'im_class'):
             klass = func.im_class
             module.append(klass.__name__)
     if os.name == 'nt' and win_characters:
         # Stupid windows can't encode certain characters in filenames
-        import urllib
-        for char in ('<', '>', '!', ':'):
-            name = name.replace(char, urllib.quote(char))
+        name = _clean_win_chars(name)
+        module = [_clean_win_chars(s) for s in module]
     return module, name
 
 
@@ -126,7 +137,7 @@ def filter_args(func, ignore_lst, *args, **kwargs):
         func: callable
             Function giving the argument specification
         ignore_lst: list of strings
-            List of arguments to ignore (either a name of an argument 
+            List of arguments to ignore (either a name of an argument
             in the function spec, or '*', or '**')
         *args: list
             Positional arguments passed to the function.
@@ -146,18 +157,18 @@ def filter_args(func, ignore_lst, *args, **kwargs):
         raise ValueError('ignore_lst must be a list of parameters to ignore '
             '%s (type %s) was given' % (ignore_lst, type(ignore_lst)))
     # Special case for functools.partial objects
-    if (not inspect.ismethod(func) and not inspect.isfunction(func)): 
+    if (not inspect.ismethod(func) and not inspect.isfunction(func)):
         if ignore_lst:
             warnings.warn('Cannot inspect object %s, ignore list will '
                 'not work.' % func, stacklevel=2)
-        return {'*':args, '**':kwargs}
+        return {'*': args, '**': kwargs}
     arg_spec = inspect.getargspec(func)
     # We need to if/them to account for different versions of Python
     if hasattr(arg_spec, 'args'):
-        arg_names    = arg_spec.args
+        arg_names = arg_spec.args
         arg_defaults = arg_spec.defaults
         arg_keywords = arg_spec.keywords
-        arg_varargs  = arg_spec.varargs
+        arg_varargs = arg_spec.varargs
     else:
         arg_names, arg_varargs, arg_keywords, arg_defaults = arg_spec
     arg_defaults = arg_defaults or {}
@@ -165,7 +176,7 @@ def filter_args(func, ignore_lst, *args, **kwargs):
         # First argument is 'self', it has been removed by Python
         # we need to add it back:
         args = [func.im_self, ] + args
-    # XXX: Maybe I need an inspect.isbuiltin to detect C-level methods, such 
+    # XXX: Maybe I need an inspect.isbuiltin to detect C-level methods, such
     # as on ndarrays.
 
     _, name = get_func_name(func, resolv_alias=False)
@@ -186,16 +197,15 @@ def filter_args(func, ignore_lst, *args, **kwargs):
                     # Missing argument
                     raise ValueError('Wrong number of arguments for %s%s:\n'
                                      '     %s(%s, %s) was called.'
-                        % (name, 
+                        % (name,
                            inspect.formatargspec(*inspect.getargspec(func)),
                            name,
                            repr(args)[1:-1],
-                           ', '.join('%s=%s' % (k, v) 
+                           ', '.join('%s=%s' % (k, v)
                                     for k, v in kwargs.iteritems())
                            )
                         )
 
-
     varkwargs = dict()
     for arg_name, arg_value in kwargs.iteritems():
         if arg_name in arg_dict:
@@ -209,7 +219,7 @@ def filter_args(func, ignore_lst, *args, **kwargs):
     if arg_keywords is not None:
         arg_dict['**'] = varkwargs
     if arg_varargs is not None:
-        varargs = args[arg_position+1:]
+        varargs = args[arg_position + 1:]
         arg_dict['*'] = varargs
 
     # Now remove the arguments to be ignored
@@ -218,7 +228,7 @@ def filter_args(func, ignore_lst, *args, **kwargs):
             arg_dict.pop(item)
         else:
             raise ValueError("Ignore list: argument '%s' is not defined for "
-            "function %s%s" % 
+            "function %s%s" %
                             (item, name,
                              inspect.formatargspec(arg_names,
                                                    arg_varargs,
@@ -226,5 +236,4 @@ def filter_args(func, ignore_lst, *args, **kwargs):
                                                    arg_defaults,
                                                    )))
     # XXX: Return a sorted list of pairs?
-    return arg_dict 
-
+    return arg_dict
