@@ -1,17 +1,27 @@
 """
-Fast cryptographic hash of Python objects, with a special case for fast 
+Fast cryptographic hash of Python objects, with a special case for fast
 hashing of numpy arrays.
 """
 
-# Author: Gael Varoquaux <gael dot varoquaux at normalesup dot org> 
+# Author: Gael Varoquaux <gael dot varoquaux at normalesup dot org>
 # Copyright (c) 2009 Gael Varoquaux
 # License: BSD Style, 3 clauses.
 
 import pickle
 import hashlib
 import sys
-import cStringIO
 import types
+import struct
+
+if sys.version_info[0] == 3:
+    # in python3, StringIO does not accept binary data
+    # see http://packages.python.org/six/
+    import io
+    StringIO = io.BytesIO
+else:
+    import cStringIO
+    StringIO = cStringIO.StringIO
+
 
 class Hasher(pickle.Pickler):
     """ A subclass of pickler, to do cryptographic hashing, rather than
@@ -19,7 +29,7 @@ class Hasher(pickle.Pickler):
     """
 
     def __init__(self, hash_name='md5'):
-        self.stream = cStringIO.StringIO()
+        self.stream = StringIO()
         pickle.Pickler.__init__(self, self.stream, protocol=2)
         # Initialise the hash obj
         self._hash = hashlib.new(hash_name)
@@ -41,6 +51,33 @@ class Hasher(pickle.Pickler):
             obj = (func_name, inst, cls)
         pickle.Pickler.save(self, obj)
 
+    def save_global(self, obj, name=None, pack=struct.pack):
+        # We have to override this method in order to deal with objects
+        # defined interactively in IPython that are not injected in
+        # __main__
+        module = getattr(obj, "__module__", None)
+        if module == '__main__':
+            my_name = name
+            if my_name is None:
+                my_name = obj.__name__
+            mod = sys.modules[module]
+            if not hasattr(mod, my_name):
+                # IPython doesn't inject the variables define
+                # interactively in __main__
+                setattr(mod, my_name, obj)
+        pickle.Pickler.save_global(self, obj, name=name, pack=struct.pack)
+
+    dispatch = pickle.Pickler.dispatch.copy()
+    # builtin
+    dispatch[type(len)] = save_global
+    # type
+    dispatch[type(object)] = save_global
+    # classobj
+    dispatch[type(pickle.Pickler)] = save_global
+    # function
+    dispatch[type(pickle.dump)] = save_global
+
+
 class NumpyHasher(Hasher):
     """ Special case the hasher for when numpy is loaded.
     """
@@ -76,7 +113,7 @@ class NumpyHasher(Hasher):
                 # XXX: There might be a more efficient way of doing this
                 self._hash.update(self.np.getbuffer(obj.flatten()))
 
-            # We store the class, to be able to distinguish between 
+            # We store the class, to be able to distinguish between
             # Objects with the same binary content, but different
             # classes.
             if self.coerce_mmap and isinstance(obj, self.np.memmap):
@@ -86,7 +123,7 @@ class NumpyHasher(Hasher):
                 klass = self.np.ndarray
             else:
                 klass = obj.__class__
-            # We also return the dtype and the shape, to distinguish 
+            # We also return the dtype and the shape, to distinguish
             # different views on the same data with different dtypes.
 
             # The object will be pickled by the pickler hashed at the end.
@@ -95,14 +132,14 @@ class NumpyHasher(Hasher):
 
 
 def hash(obj, hash_name='md5', coerce_mmap=False):
-    """ Quick calculation of a hash to identify uniquely Python objects 
+    """ Quick calculation of a hash to identify uniquely Python objects
         containing numpy arrays.
 
-    
+
         Parameters
         -----------
         hash_name: 'md5' or 'sha1'
-            Hashing algorithm used. sha1 is supposedly safer, but md5 is 
+            Hashing algorithm used. sha1 is supposedly safer, but md5 is
             faster.
         coerce_mmap: boolean
             Make no difference between np.memmap and np.ndarray
@@ -112,4 +149,3 @@ def hash(obj, hash_name='md5', coerce_mmap=False):
     else:
         hasher = Hasher(hash_name=hash_name)
     return hasher.hash(obj)
-
