@@ -7,6 +7,8 @@ Helpers for embarrassingly parallel code.
 
 import os
 import sys
+import warnings
+from math import sqrt
 import functools
 import time
 import threading
@@ -31,7 +33,38 @@ from .logger import Logger, short_format_time
 from .my_exceptions import TransportableException, _mk_exception
 
 
-################################################################################
+###############################################################################
+# CPU that works also when multiprocessing is not installed (python2.5)
+def cpu_count():
+    """ Return the number of CPUs.
+    """
+    if multiprocessing is None:
+        return 1
+    return multiprocessing.cpu_count()
+
+
+###############################################################################
+# For verbosity
+
+def _verbosity_filter(index, verbose):
+    """ Returns False for indices increasingly appart, the distance
+        depending on the value of verbose.
+
+        We use a lag increasing as the square of index
+    """
+    if not verbose:
+        return True
+    elif verbose > 10:
+        return False
+    if index == 0:
+        return False
+    verbose = .5 * (11 - verbose) ** 2
+    scale = sqrt(index / verbose)
+    next_scale = sqrt((index + 1) / verbose)
+    return (int(next_scale) == int(scale))
+
+
+###############################################################################
 class WorkerInterrupt(Exception):
     """ An exception that is not KeyboardInterrupt to allow subprocesses
         to be interrupted.
@@ -39,18 +72,16 @@ class WorkerInterrupt(Exception):
     pass
 
 
-################################################################################
+###############################################################################
 class SafeFunction(object):
     """ Wraps a function to make it exception with full traceback in
         their representation.
-        Useful for parallel computing with multiprocessing, for which 
+        Useful for parallel computing with multiprocessing, for which
         exceptions cannot be captured.
     """
-
     def __init__(self, func):
         self.func = func
 
-
     def __call__(self, *args, **kwargs):
         try:
             return self.func(*args, **kwargs)
@@ -66,7 +97,7 @@ class SafeFunction(object):
             raise TransportableException(text, e_type)
 
 
-################################################################################
+###############################################################################
 def delayed(function):
     """ Decorator used to capture the arguments of a function.
     """
@@ -83,22 +114,22 @@ def delayed(function):
     return delayed_function
 
 
-################################################################################
+###############################################################################
 class ImmediateApply(object):
     """ A non-delayed apply function.
     """
-    def __init__ (self, func, args, kwargs):
+    def __init__(self, func, args, kwargs):
         # Don't delay the application, to avoid keeping the input
         # arguments in memory
         self.results = func(*args, **kwargs)
 
-    def get (self):
+    def get(self):
         return self.results
 
 
-################################################################################
+###############################################################################
 class CallBack(object):
-    """ Callback used by parallel: it is used for progress reporting, and 
+    """ Callback used by parallel: it is used for progress reporting, and
         to add data to be processed
     """
     def __init__(self, index, parallel):
@@ -106,42 +137,12 @@ class CallBack(object):
         self.index = index
 
     def __call__(self, out):
-        if self.parallel.verbose:
-            self.print_progress()
+        self.parallel.print_progress(self.index)
         if self.parallel._iterable:
             self.parallel.dispatch_next()
 
-    def print_progress(self):
-        # XXX: Not using the logger framework: need to
-        # learn to use logger better.
-        n_jobs = len(self.parallel._pool._pool)
-        if self.parallel.n_dispatched > 2*n_jobs:
-            # Report less often
-            if not self.index % n_jobs == 0:
-                return
-        elapsed_time = time.time() - self.parallel._start_time
-        remaining_time = (elapsed_time/(self.index + 1)*
-                    (self.parallel.n_dispatched - self.index - 1.))
-        if self.parallel._iterable:
-            # The object is still building it's job list
-            total = "%3i+" % self.parallel.n_dispatched
-        else:
-            total = "%3i " % self.parallel.n_dispatched
 
-        if self.parallel.verbose < 50:
-            writer = sys.stderr.write
-        else:
-            writer = sys.stdout.write
-        writer('[%s]: Done %3i out of %s |elapsed: %s remaining: %s\n'
-                % (self.parallel,
-                    self.index+1, 
-                    total, 
-                    short_format_time(elapsed_time),
-                    short_format_time(remaining_time),
-                    ))
-
-
-################################################################################
+###############################################################################
 class Parallel(Logger):
     ''' Helper class for readable parallel mapping.
 
@@ -150,11 +151,14 @@ class Parallel(Logger):
         n_jobs: int
             The number of jobs to use for the computation. If -1 all CPUs
             are used. If 1 is given, no parallel computing code is used
-            at all, which is useful for debuging.
+            at all, which is useful for debuging. For n_jobs below -1,
+            (n_cpus + 1 - n_jobs) are used. Thus for n_jobs = -2, all
+            CPUs but one are used.
         verbose: int, optional
-            The verbosity level. If 1 is given, the elapsed time as well
-            as the estimated remaining time are displayed. Above 100, the
-            output is sent to stdout.
+            The verbosity level: if non zero, progress messages are
+            printed. Above 50, the output is sent to stdout.
+            The frequency of the messages increases with the verbosity level.
+            If it more than 10, all iterations are reported.
         pre_dispatch: {'all', integer, or expression, as in '3*n_jobs'}
             The amount of jobs to be pre-dispatched. Default is 'all',
             but it may be memory consuming, for instance if each job
@@ -203,17 +207,18 @@ class Parallel(Logger):
         (0.0, 0.5, 0.0, 0.5, 0.0, 0.5, 0.0, 0.5, 0.0, 0.5)
         >>> i
         (0.0, 0.0, 1.0, 1.0, 2.0, 2.0, 3.0, 3.0, 4.0, 4.0)
-       
-        The progress meter::
+
+        The progress meter: the higher the value of `verbose`, the more
+        messages::
 
             >>> from time import sleep
             >>> from joblib import Parallel, delayed
-            >>> r = Parallel(n_jobs=2, verbose=1)(delayed(sleep)(.1) for _ in range(10)) #doctest: +SKIP
-            [Parallel(n_jobs=2)]: Done   1 out of  10 |elapsed:    0.1s remaining:    0.9s
-            [Parallel(n_jobs=2)]: Done   3 out of  10 |elapsed:    0.2s remaining:    0.5s
-            [Parallel(n_jobs=2)]: Done   5 out of  10 |elapsed:    0.3s remaining:    0.3s
-            [Parallel(n_jobs=2)]: Done   7 out of  10 |elapsed:    0.4s remaining:    0.2s
-            [Parallel(n_jobs=2)]: Done   9 out of  10 |elapsed:    0.5s remaining:    0.1s
+            >>> r = Parallel(n_jobs=2, verbose=5)(delayed(sleep)(.1) for _ in range(10)) #doctest: +SKIP
+            [Parallel(n_jobs=2)]: Done   1 out of  10 | elapsed:    0.1s remaining:    0.9s
+            [Parallel(n_jobs=2)]: Done   3 out of  10 | elapsed:    0.2s remaining:    0.5s
+            [Parallel(n_jobs=2)]: Done   6 out of  10 | elapsed:    0.3s remaining:    0.2s
+            [Parallel(n_jobs=2)]: Done   9 out of  10 | elapsed:    0.5s remaining:    0.1s
+            [Parallel(n_jobs=2)]: Done  10 out of  10 | elapsed:    0.5s finished
 
         Traceback example, note how the line of the error is indicated
         as well as the values of the parameter passed to the function that
@@ -225,7 +230,7 @@ class Parallel(Logger):
          >>> Parallel(n_jobs=2)(delayed(atoi)(n) for n in ('1', '300', 30)) #doctest: +SKIP
          #...
          ---------------------------------------------------------------------------
-         Sub-process traceback: 
+         Sub-process traceback:
          ---------------------------------------------------------------------------
          TypeError                                          Fri Jul  2 20:32:05 2010
          PID: 4151                                     Python 2.6.5: /usr/bin/python
@@ -234,14 +239,14 @@ class Parallel(Logger):
              398     is chosen from the leading characters of s, 0 for octal, 0x or
              399     0X for hexadecimal.  If base is 16, a preceding 0x or 0X is
              400     accepted.
-             401 
+             401
              402     """
          --> 403     return _int(s, base)
-             404 
-             405 
+             404
+             405
              406 # Convert string to long integer
              407 def atol(s, base=10):
-         
+
          TypeError: int() can't convert non-string with explicit base
          ___________________________________________________________________________
 
@@ -249,7 +254,7 @@ class Parallel(Logger):
         data is generated on the fly. Note how the producer is first
         called a 3 times before the parallel loop is initiated, and then
         called to generate new data on the fly. In this case the total
-        number of iterations reported is underestimated::
+        number of iterations cannot be reported in the progress messages::
 
          >>> from math import sqrt
          >>> from joblib import Parallel, delayed
@@ -258,40 +263,43 @@ class Parallel(Logger):
          ...     for i in range(6):
          ...         print 'Produced %s' % i
          ...         yield i
-         
+
          >>> out = Parallel(n_jobs=2, verbose=100, pre_dispatch='1.5*n_jobs')(
-         ...                         delayed(sqrt)(i) for i in producer()) #doctest: +ELLIPSIS
+         ...                         delayed(sqrt)(i) for i in producer()) #doctest: +SKIP
          Produced 0
          Produced 1
          Produced 2
-         [Parallel(n_jobs=2)]: Done   1 out of   3+ |elapsed:   ...s remaining:   ...s
+         [Parallel(n_jobs=2)]: Done   1 jobs       | elapsed:    0.0s
          Produced 3
-         [Parallel(n_jobs=2)]: Done ... out of   4+ |elapsed:   ...s remaining:   ...s
-         ...
-
+         [Parallel(n_jobs=2)]: Done   2 jobs       | elapsed:    0.0s
+         Produced 4
+         [Parallel(n_jobs=2)]: Done   3 jobs       | elapsed:    0.0s
+         Produced 5
+         [Parallel(n_jobs=2)]: Done   4 jobs       | elapsed:    0.0s
+         [Parallel(n_jobs=2)]: Done   5 out of   6 | elapsed:    0.0s remaining:    0.0s
+         [Parallel(n_jobs=2)]: Done   6 out of   6 | elapsed:    0.0s finished
     '''
     def __init__(self, n_jobs=None, verbose=0, pre_dispatch='all'):
         self.verbose = verbose
-        self.n_jobs  = n_jobs
-        self.pre_dispatch   = pre_dispatch
-        self._pool   = None
-        # Not starting the pool in the __init__ is a design decision, to
-        # be able to close it ASAP, and not burden the user with closing
-        # it.
+        self.n_jobs = n_jobs
+        self.pre_dispatch = pre_dispatch
+        self._pool = None
+        # Not starting the pool in the __init__ is a design decision, to be
+        # able to close it ASAP, and not burden the user with closing it.
         self._output = None
-        self._jobs   = list()
-
+        self._jobs = list()
 
     def dispatch(self, func, args, kwargs):
-        """ Queue the function for computing, with or without multiprocessing 
+        """ Queue the function for computing, with or without multiprocessing
         """
         if self._pool is None:
             job = ImmediateApply(func, args, kwargs)
-            if self.verbose:
-                print '[%s]: Done job %3i | elapsed: %s' % (
-                        self, len(self._jobs)+1,
-                        short_format_time(time.time() - self._start_time)
-                    )
+            index = len(self._jobs)
+            if not _verbosity_filter(index, self.verbose):
+                self._print('Done %3i jobs       | elapsed: %s',
+                        (index + 1,
+                            short_format_time(time.time() - self._start_time)
+                        ))
             self._jobs.append(job)
             self.n_dispatched += 1
         else:
@@ -307,14 +315,13 @@ class Parallel(Logger):
             finally:
                 self._lock.release()
 
-
     def dispatch_next(self):
         """ Dispatch more data for parallel processing
         """
         self._dispatch_amount += 1
         while self._dispatch_amount:
             try:
-                # XXX: possible race condition shuffling the order of 
+                # XXX: possible race condition shuffling the order of
                 # dispatchs in the next two lines.
                 func, args, kwargs = self._iterable.next()
                 self.dispatch(func, args, kwargs)
@@ -327,6 +334,58 @@ class Parallel(Logger):
                 self._iterable = None
                 return
 
+    def _print(self, msg, msg_args):
+        """ Display the message on stout or stderr depending on verbosity
+        """
+        # XXX: Not using the logger framework: need to
+        # learn to use logger better.
+        if not self.verbose:
+            return
+        if self.verbose < 50:
+            writer = sys.stderr.write
+        else:
+            writer = sys.stdout.write
+        msg = msg % msg_args
+        writer('[%s]: %s\n' % (self, msg))
+
+    def print_progress(self, index):
+        """Display the process of the parallel execution only a fraction
+           of time, controled by self.verbose.
+        """
+        if not self.verbose:
+            return
+        elapsed_time = time.time() - self._start_time
+
+        # This is heuristic code to print only 'verbose' times a messages
+        # The challenge is that we may not know the queue length
+        if self._iterable:
+            if _verbosity_filter(index, self.verbose):
+                return
+            self._print('Done %3i jobs       | elapsed: %s',
+                        (index + 1,
+                         short_format_time(elapsed_time),
+                        ))
+        else:
+            # We are finished dispatching
+            queue_length = self.n_dispatched
+            # We always display the first loop
+            if not index == 0:
+                # Display depending on the number of remaining items
+                # A message as soon as we finish dispatching, cursor is 0
+                cursor = (queue_length - index + 1
+                          - self._pre_dispatch_amount)
+                frequency = (queue_length // self.verbose) + 1
+                is_last_item = (index + 1 == queue_length)
+                if (is_last_item or cursor % frequency):
+                    return
+            remaining_time = (elapsed_time / (index + 1) *
+                        (self.n_dispatched - index - 1.))
+            self._print('Done %3i out of %3i | elapsed: %s remaining: %s',
+                        (index + 1,
+                         queue_length,
+                         short_format_time(elapsed_time),
+                         short_format_time(remaining_time),
+                        ))
 
     def retrieve(self):
         self._output = list()
@@ -341,7 +400,7 @@ class Parallel(Logger):
             try:
                 self._output.append(job.get())
             except tuple(self.exceptions), exception:
-                if isinstance(exception, 
+                if isinstance(exception,
                         (KeyboardInterrupt, WorkerInterrupt)):
                     # We have captured a user interruption, clean up
                     # everything
@@ -350,17 +409,14 @@ class Parallel(Logger):
                         self._pool.terminate()
                     raise exception
                 elif isinstance(exception, TransportableException):
-                    # Capture exception to add information on 
-                    # the local stack in addition to the distant
-                    # stack
-                    this_report = format_outer_frames(
-                                            context=10,
-                                            stack_start=1,
-                                            )
+                    # Capture exception to add information on the local stack
+                    # in addition to the distant stack
+                    this_report = format_outer_frames(context=10,
+                                                      stack_start=1)
                     report = """Multiprocessing exception:
 %s
 ---------------------------------------------------------------------------
-Sub-process traceback: 
+Sub-process traceback:
 ---------------------------------------------------------------------------
 %s""" % (
                             this_report,
@@ -371,13 +427,12 @@ Sub-process traceback:
                     raise exception_type(report)
                 raise exception
 
-
     def __call__(self, iterable):
         if self._jobs:
             raise ValueError('This Parallel instance is already running')
         n_jobs = self.n_jobs
-        if n_jobs == -1 and multiprocessing is not None:
-            n_jobs = multiprocessing.cpu_count()
+        if n_jobs < 0 and multiprocessing is not None:
+            n_jobs = max(multiprocessing.cpu_count() + 1 + n_jobs, 1)
 
         # The list of exceptions that we will capture
         self.exceptions = [TransportableException]
@@ -385,21 +440,30 @@ Sub-process traceback:
             n_jobs = 1
             self._pool = None
         else:
-            self._pool = multiprocessing.Pool(n_jobs)
-            self._lock = threading.Lock()
-            # We are using multiprocessing, we also want to capture
-            # KeyboardInterrupts
-            self.exceptions.extend([KeyboardInterrupt, WorkerInterrupt])
+            if multiprocessing.current_process()._daemonic:
+                # Daemonic processes cannot have children
+                n_jobs = 1
+                self._pool = None
+                warnings.warn(
+                    'Parallel loops cannot be nested, setting n_jobs=1',
+                    stacklevel=2)
+            else:
+                self._pool = multiprocessing.Pool(n_jobs)
+                self._lock = threading.Lock()
+                # We are using multiprocessing, we also want to capture
+                # KeyboardInterrupts
+                self.exceptions.extend([KeyboardInterrupt, WorkerInterrupt])
 
         if self.pre_dispatch == 'all' or n_jobs == 1:
             self._iterable = None
+            self._pre_dispatch_amount = 0
         else:
             self._iterable = iterable
             self._dispatch_amount = 0
             pre_dispatch = self.pre_dispatch
             if hasattr(pre_dispatch, 'endswith'):
                 pre_dispatch = eval(pre_dispatch)
-            pre_dispatch = int(pre_dispatch)
+            self._pre_dispatch_amount = pre_dispatch = int(pre_dispatch)
             iterable = itertools.islice(iterable, pre_dispatch)
 
         self._start_time = time.time()
@@ -407,8 +471,16 @@ Sub-process traceback:
         try:
             for function, args, kwargs in iterable:
                 self.dispatch(function, args, kwargs)
-            
+
             self.retrieve()
+            # Make sure that we get a last message telling us we are done
+            elapsed_time = time.time() - self._start_time
+            self._print('Done %3i out of %3i | elapsed: %s finished',
+                        (len(self._output),
+                         len(self._output),
+                            short_format_time(elapsed_time)
+                        ))
+
         finally:
             if n_jobs > 1:
                 self._pool.close()
@@ -418,12 +490,5 @@ Sub-process traceback:
         self._output = None
         return output
 
-
     def __repr__(self):
-        return '%s(n_jobs=%s)' % (
-                    self.__class__.__name__,
-                    self.n_jobs,
-                )
-
-
-
+        return '%s(n_jobs=%s)' % (self.__class__.__name__, self.n_jobs)
